# # program to convert fits data cube to fits and save that dta cube to the 
# # final one

import numpy as np
from astropy.io import fits
from pathlib import Path
import matplotlib.pyplot as plt
import glob
import os
import datetime

# def fits_to_fits(parent_folder):
parent_folder = '/data/solar_data/raw/2023/06/01/'
sub_folder = 'DARK/'
# fits_name = '2022 June 02 09_44_28 10.fits'
fits_pattern = '*.fits'
# folder_path_s = os.path.join(parent_folder,fits_name)
folder_path_s = os.path.join(parent_folder,sub_folder,fits_pattern)
fits_files =  glob.glob(folder_path_s)

# open the fits data cube
# loop through each fits files
for fits_file in fits_files:
    # open the fita data cube
    data_cube = fits.open(fits_file)
    header =  data_cube[0].header
    date = header['DATE-OBS']
# Loop through each slice of the data cube and save it as a separate fits file
    for i, data_slice in enumerate(data_cube[0].data, start=1):
        # Get the header for this slice from the data cube
        header = data_cube[0].header
        # Convert the header DATE-OBS value to datetime object
        date_obs_str = header['DATE-OBS']
        date_obs_time = datetime.datetime.strptime(date_obs_str, '%Y-%m-%dT%H:%M:%S')

        # Add cadence to datetime object
        # cadence = int(header['PI CAMERA SHUTTERTIMING DELAYRESOLUTION'])/1000
        cadence = 1 #cadence is kept one sec for Dark and Flat
        date_obs_time_cadence = date_obs_time + datetime.timedelta(seconds=cadence)

        # Convert back to string format
        date_obs_str_cadence = date_obs_time_cadence.strftime('%Y-%m-%dT%H:%M:%S')

        # Update header with new DATE-OBS value
        header['DATE-OBS'] = date_obs_str_cadence
        date_only = date.split("T")[0]
        fitsfile=f'D-{header["DATE-OBS"]}-HA.fits'
        fitsname = fitsfile
        params={'rootfolder': '/data/solar_data/processed/'}
        if Path(params["rootfolder"]).exists():
        # if params['rootfolder'] is None:
            dcf = fitsname.split('-')
            dcf_date = dcf[3].split('T')[0]
            date_obs = datetime.datetime.strptime(date_only, '%Y-%m-%d')
            #currentfolder=f'{params["rootfolder"]}{params["destfolder"]}{dcf[1]}/{dcf[1]}{dcf[2]}{dcf_date}/'
            currentfolder=f'{params["rootfolder"]}{dcf[1]}/{date_obs.strftime("%b").capitalize()}/{dcf[1]}{dcf[2]}{dcf_date}/'
            if not Path(currentfolder).exists():
                Path(currentfolder).mkdir(parents=True,exist_ok=True)
            # return f'{currentfolder}{fitsname}'
            fits.writeto(os.path.join(currentfolder, fitsfile),data_slice,header,overwrite=True)

# # def fits_to_fits(parent_folder):
sub_folder = 'FLAT/'
# fits_name = '2022 June 02 09_44_28 10.fits'
fits_pattern = '*.fits'
# folder_path_s = os.path.join(parent_folder,fits_name)
folder_path_s = os.path.join(parent_folder,sub_folder,fits_pattern)
fits_files =  glob.glob(folder_path_s)

# open the fits data cube
# loop through each fits files
for fits_file in fits_files:
    # open the fita data cube
    data_cube = fits.open(fits_file)
    header =  data_cube[0].header
    date = header['DATE-OBS']
# Loop through each slice of the data cube and save it as a separate fits file
    for i, data_slice in enumerate(data_cube[0].data, start=1):
        # Get the header for this slice from the data cube
        header = data_cube[0].header
        # Convert the header DATE-OBS value to datetime object
        date_obs_str = header['DATE-OBS']
        date_obs_time = datetime.datetime.strptime(date_obs_str, '%Y-%m-%dT%H:%M:%S')

        # Add cadence to datetime object
        # cadence = int(header['PI CAMERA SHUTTERTIMING DELAYRESOLUTION'])/1000
        cadence = 1 #cadence is kept one sec for Dark and Flat
        date_obs_time_cadence = date_obs_time + datetime.timedelta(seconds=cadence)

        # Convert back to string format
        date_obs_str_cadence = date_obs_time_cadence.strftime('%Y-%m-%dT%H:%M:%S')

        # Update header with new DATE-OBS value
        header['DATE-OBS'] = date_obs_str_cadence
        date_only = date.split("T")[0]
        fitsfile=f'F-{header["DATE-OBS"]}-HA.fits'
        fitsname = fitsfile
        params={'rootfolder': '/data/solar_data/processed/'}
        if Path(params["rootfolder"]).exists():
        # if params['rootfolder'] is None:
            dcf = fitsname.split('-')
            dcf_date = dcf[3].split('T')[0]
            date_obs = datetime.datetime.strptime(date_only, '%Y-%m-%d')
        #currentfolder=f'{params["rootfolder"]}{params["destfolder"]}{dcf[1]}/{dcf[1]}{dcf[2]}{dcf_date}/'
            currentfolder=f'{params["rootfolder"]}{dcf[1]}/{date_obs.strftime("%b").capitalize()}/{dcf[1]}{dcf[2]}{dcf_date}/'
            if not Path(currentfolder).exists():
                Path(currentfolder).mkdir(parents=True,exist_ok=True)
        # return f'{currentfolder}{fitsname}'
            fits.writeto(os.path.join(currentfolder, fitsfile),data_slice,header,overwrite=True)

# def fits_to_fits(parent_folder):
sub_folder = 'HC/'
# fits_name = '2022 June 02 09_44_28 10.fits'
fits_pattern = '*.fits'
# folder_path_s = os.path.join(parent_folder,fits_name)
folder_path_s = os.path.join(parent_folder,sub_folder,fits_pattern)
fits_files =  glob.glob(folder_path_s)

# open the fits data cube
# loop through each fits files
for fits_file in fits_files:
    # open the fita data cube
    data_cube = fits.open(fits_file)
    header =  data_cube[0].header
    date = header['DATE-OBS']
    date_only = date.split("T")[0]
    # Loop through each slice of the data cube and save it as a separate fits file
    for i, data_slice in enumerate(data_cube[0].data, start=1):
        # Get the header for this slice from the data cube
        header = data_cube[0].header
    #   Convert the header DATE-OBS value to datetime object
        date_obs_str = header['DATE-OBS']
        date_obs_time = datetime.datetime.strptime(date_obs_str, '%Y-%m-%dT%H:%M:%S')

    #   Add cadence to datetime object
    #   cadence = int(header['PI CAMERA SHUTTERTIMING DELAYRESOLUTION'])/1000
        cadence = 5 #cadence is kept five sec for HC
        date_obs_time_cadence = date_obs_time + datetime.timedelta(seconds=cadence)

    #   Convert back to string format
        date_obs_str_cadence = date_obs_time_cadence.strftime('%Y-%m-%dT%H:%M:%S')

    #   Update header with new DATE-OBS value
        header['DATE-OBS'] = date_obs_str_cadence

        fitsfile=f'S-{header["DATE-OBS"]}-HA.fits'
        fitsname = fitsfile
        params={'rootfolder': '/data/solar_data/processed/'}
        if Path(params["rootfolder"]).exists():
    #   if params['rootfolder'] is None:
            dcf = fitsname.split('-')
            dcf_date = dcf[3].split('T')[0]
            date_obs = datetime.datetime.strptime(date_only, '%Y-%m-%d')
            #currentfolder=f'{params["rootfolder"]}{params["destfolder"]}{dcf[1]}/{dcf[1]}{dcf[2]}{dcf_date}/'
            currentfolder=f'{params["rootfolder"]}{dcf[1]}/{date_obs.strftime("%b").capitalize()}/{dcf[1]}{dcf[2]}{dcf_date}/'
            if not Path(currentfolder).exists():
                Path(currentfolder).mkdir(parents=True,exist_ok=True)
            # return f'{currentfolder}{fitsname}'
            fits.writeto(os.path.join(currentfolder, fitsfile),data_slice,header,overwrite=True)

# def fits_to_fits(parent_folder):
sub_folder = 'LC/'
# fits_name = '2022 June 02 09_44_28 10.fits'
fits_pattern = '*.fits'
# folder_path_s = os.path.join(parent_folder,fits_name)
folder_path_s = os.path.join(parent_folder,sub_folder,fits_pattern)
fits_files =  glob.glob(folder_path_s)

# open the fits data cube
# loop through each fits files
for fits_file in fits_files:
    # open the fita data cube
    data_cube = fits.open(fits_file)
    header =  data_cube[0].header

# Loop through each slice of the data cube and save it as a separate fits file
    for i, data_slice in enumerate(data_cube[0].data, start=1):
        # Get the header for this slice from the data cube
        header = data_cube[0].header
    #   Convert the header DATE-OBS value to datetime object
        date_obs_str = header['DATE-OBS']
        date_obs_time = datetime.datetime.strptime(date_obs_str, '%Y-%m-%dT%H:%M:%S')

    #   Add cadence to datetime object
    #   cadence = int(header['PI CAMERA SHUTTERTIMING DELAYRESOLUTION'])/1000
        cadence =  30 #ensure to keep cadence of 30 sec for Low cadence data
        date_obs_time_cadence = date_obs_time + datetime.timedelta(seconds=cadence)

    #   Convert back to string format
        date_obs_str_cadence = date_obs_time_cadence.strftime('%Y-%m-%dT%H:%M:%S')

    #   Update header with new DATE-OBS value
        header['DATE-OBS'] = date_obs_str_cadence

        fitsfile=f'S-{header["DATE-OBS"]}-HA.fits'
        fitsname = fitsfile
        params={'rootfolder': '/data/solar_data/processed/'}
        if Path(params["rootfolder"]).exists():
    # if params['rootfolder'] is None:
            dcf = fitsname.split('-')
            dcf_date = dcf[3].split('T')[0]
            date_obs = datetime.datetime.strptime(date_only, '%Y-%m-%d')
        #   currentfolder=f'{params["rootfolder"]}{params["destfolder"]}{dcf[1]}/{dcf[1]}{dcf[2]}{dcf_date}/'
            currentfolder=f'{params["rootfolder"]}{dcf[1]}/{date_obs.strftime("%b").capitalize()}/{dcf[1]}{dcf[2]}{dcf_date}/'
            if not Path(currentfolder).exists():
                 Path(currentfolder).mkdir(parents=True,exist_ok=True)
         # return f'{currentfolder}{fitsname}'
            fits.writeto(os.path.join(currentfolder, fitsfile),data_slice,header,overwrite=True)



# This is a default configuration value file, 
# Where you can hardcode the following variables


# INPUTS ---------------------------------------------------------
rootfolder='/data/solar_data/'
processed='processed/'
thumbnails='Thumbnails/'

processedfolder=f'{rootfolder}{processed}'
thumbnails_root=f'{rootfolder}{thumbnails}'       # Thumbnails in the root folder has all thumbnails from processed fits folder


rawfolder=f'{rootfolder}raw/'                           # Parent RAW Folder

hc=f'HC/'                                               # high cadence files
hc_initial='S'
hc_cadence='5'

lc=f'LC/'                                               # low cadence files
lc_initial='S'
lc_cadence='30'

dark=f'DARK/'                                           # DARK frame files
dark_initial='D'
dark_cadence='1'

flat=f'FLAT/'                                           # FLAT frame files
flat_initial='F'
flat_cadence='1'

corrupted_folder='corrupt/'                   # The folder name for corrupted raw files inside the current directory of processed fits file when conversion fails.

# SETTINGS --------------------------------------------------------
usemagick=True
uselogging=False
logfile='scoobi.logs'
logfolder='.'

# HEADER DEFAULT --------------------------------------------------
telescope_name='HA'

# -----------------------------------------------------------------
config = {key: value for key, value in locals().items() if not key.startswith('__')}



# Solar Conventionality-based Organizing Observation data ( SCOOBI )

import exifread, subprocess, argparse
from glob import glob
from pathlib import Path
from astropy.io import fits
from astropy.time import Time
import astropy.units as u
from collections import defaultdict
import csv
import datetime
import shutil
from scoobi.config import usemagick, thumbnails, thumbnails_root, rootfolder
from scoobi.config import  dark,dark_cadence,dark_initial,flat,flat_cadence,flat_initial,hc,hc_cadence,hc_initial,lc,lc_cadence,lc_initial


class SCOOBI():
    def __init__():
        pass

# Helper
def read_tif(tiffile):
    with open(tiffile, 'rb') as f:
        tags=exifread.process_file(f)
    return (tags)

def _vitals_check():
    """
    Check if magick is installed or not. Returns 0 if exists else non-zero int.
    """
    a=subprocess.call('magick -version',shell=True, stdout=subprocess.DEVNULL)
    return {'magick':a}

def logger():
    pass

# Folder Structure
def search_file(path='', file_format='.tif', recursive=False, dontknowfoldername=False):
    if dontknowfoldername:
        files=glob(f'{path}*/**/*{file_format}',recursive=True)
    else:
        files=glob(f'{path}**/*{file_format}',recursive=recursive)
    return files

# Process
def header_dictfromtiff(tiffile):
    """
    create dict from tiffile metadata read by exifread
    *adds some headers automatically from ImageDescription
    """
    tags=read_tif(tiffile)
    header=defaultdict(list)

    t=Time.strptime(str(tags['Image DateTime'].values),'%m/%d/%Y %I:%M:%S.%f %p',scale='utc', format='fits')
    header['DATE-OBS']=t.value
    desclist = tags['Image ImageDescription'].values.split('\r\n')

    expdef=Time.strptime('00:00:00.000', '%H:%M:%S.%f')
    for desc in desclist:
        if 'Exposure' in desc:
            d=desc.split(':',1)
            exposure=Time.strptime(str(d[1])[2:-5].replace(' ',''),'%H:%M:%S.%f')
            exptime=exposure-expdef
    header['EXPTIME']=(round(float(exptime.sec),1), 'Exposure time in unit second')
    return header

def header_dictforfits(fitsfile, **kwargs):
    header_params={
        

    }
    header_params.update(kwargs)
    with fits.open(fitsfile, 'update') as f:
        for hdu in f:
            hdu.header.update(header_params)
            hdu.header['FILENAME'] = tifpath.name

def build_foldername(filename,createdir=True, **kwargs):
    """
    Builds folder name taking input of the fitsfile name i.e
    destfolder + 

    *Parameters*
    
    :filename: 
        (str) (Required)

        This is the conventional name of the fitsfile which will be used to build the folder path.
        Ex. S-2022-11-01T02:02:20.001-HA.fits

    :destfolder: 
        (str) (Optional) (Default:'')

        The destination folder name where all the files will get saved.

    *Return*
    
        (str) complete path for the file.

    """
    params={'destfolder': '',
    }
    params.update(kwargs)
    if not params['destfolder']: params['destfolder']=f"{params['fits_folder']}"
    dbf=filename.split('-') # date-wise build foldername
    dbf_date=dbf[3].split('T')[0]
    # YEAR[int4]/MONTH[str3]/yyyymmdd/S-yyyy-mm-ddTtime.fits
    mm_str=datetime.datetime.strptime(dbf[2],"%m").strftime("%b").capitalize()
    builtfolder=f'{params["destfolder"]}{dbf[1]}/{mm_str}/{dbf[1]}{dbf[2]}{dbf_date}/'
    if not Path(builtfolder).exists() and createdir:
        Path(builtfolder).mkdir(parents=True,exist_ok=True)
    return f'{builtfolder}{filename}'
        
def fits_to_fits(fitsfile_i, header=None, **kwargs):
    """
    *Parameters*
    
    :fitsfile_i:
        (str) (Required)

        Full path of the RAW FITS file as input. This is a 3D FITS file.

    :header:
        (dict) (Optional)

        If provided, it will use headers from FITS as well the supplied parameters to write to FITS.
    
    *kwargs*

    :fitsname: 
        (str) (Optional) (Default=None)

        This is the conventional name of the fitsfile which will be used to build the folder path.
        If not provided will build name according to the timestamp in the FITS header.
        Ex. S-2022-11-01T02:02:20.001-HA.fits
    
    :initial:
        (str) (Optional) (Default='S')
        Convential initial for the type of file in the output filename e.g
        - S: Science
        - F: Flat
        - D: Dark

    :destfolder: 
        (str) (Optional) (Default:see build_foldername())

        The destination folder name where all the files will get saved.

    :telescope:
        (str) (Optional) (Default:HA)
        The Telescope name is used to create the header info and filename
    """
    if header:
        pass
    params={'fitsname':None, 'telescope':'HA', 'initial':'S'}
    params.update(kwargs)
    fitsfile_o=None
    def __writefilefromhdul(hdul,date):
        """
        internal function to read hdul and date to fill headers and generate fitsfile name creating directory where necessary.
        """
        date=date.strftime('%Y-%m-%dT%H:%M:%S.%f')
        for hdu in hdul:
            hdu.header.update(header)
            hdu.header['TELESCOP']=params['telescope']
            hdu.header['FILENAME']=Path(fitsfile_i).name
            
        if params['fitsname'] is None:
            # name convention : *S + DATETIME + TELESCOPE*
            print('building foldername:')

            fitsfile_o=f"{params['initial']}-{date}-{params['telescope']}.fits"
            fitsfile_o=build_foldername(fitsfile_o, **kwargs)
        else:
            fitsfile_o=build_foldername(params['fitsname'])
        hdul.writeto(fitsfile_o, overwrite=True)
        return fitsfile_o
    
    with fits.open(fitsfile_i, 'readonly',) as hdul:
        date=hdul[0].header['DATE-OBS']
        date=datetime.datetime.strptime(date,'%Y-%m-%dT%H:%M:%S.%f')
        cadence=0
        if dark in fitsfile_i:
            params['initial']=dark_initial
            cadence=dark_cadence
        elif flat in fitsfile_i:
            params['initial']=flat_initial
            cadence=flat_cadence
        elif hc in fitsfile_i:
            params['initial']=hc_initial
            cadence=hc_cadence
        elif lc in fitsfile_i:
            params['initial']=lc_initial
            cadence=lc_cadence
            

        if len(hdul[0].data.shape)==3:                                  # checks if data is cube
            for i, hdul_slice in enumerate(hdul[0].data):
                td=datetime.timedelta(seconds=cadence*i)
                date=date+td
                fitsfile_o=__writefilefromhdul(hdul_slice,date)
        else:
            fitsfile_o=__writefilefromhdul(hdul,date)
    return fitsfile_o
    
def tif_to_fits(tiffile, magick=usemagick, header=None, **kwargs):
    """
    *Parameters*
    
    :tiffile:
        (str) (Required)

        Full path of the TIFF file that needs to be converted to FITS.

    :magick:
        (boolean) (Optional)

        If true will execute convert utility of ImageMagick to convert from tiff to fits (faster)
        Else the python way of conversion will be used.

    :header:
        (dict) (Optional)

        If provided, it will use headers from TIFF as well the supplied parameters to write to FITS.
    
    :telescope:
        (str) (Optional) (Default:HA)
        The Telescope name is used to create the header info and filename

    kwargs
    ------

    :fitsname: 
        (str) (Optional) (Default=None)

        This is the conventional name of the fitsfile which will be used to build the folder path.
        If not provided will build name according to the timestamp in the TIFF header.
        Ex. S-2022-11-01T02:02:20.001-HA.fits

    :destfolder: 
        (str) (Optional) (Default:None)

        The destination folder name where all the files will get saved.
    """
    params={'fitsname':None, 'telescope':'HA'}
    params.update(kwargs)
    tifpath = Path(tiffile)
    if header is None:
        header=header_dictfromtiff(tiffile)
    else:
        header.update(header_dictfromtiff(tiffile))
    if params['fitsname'] is None:
        # name convention : *S + DATETIME + TELESCOPE*
        fitsfile=f'S-{header["DATE-OBS"]}-{params["telescope"]}.fits'
        fitsfile=build_foldername(fitsfile, **kwargs)
    
    if magick:
        #print(tiffile)
        subprocess.run(['convert', str(tiffile), fitsfile])
        print(f'created {fitsfile}')
    else:
        pass
    with fits.open(fitsfile, 'update') as f:
        for hdu in f:
            fitspath = Path(fitsfile)
            hdu.header.update(header)
            hdu.header['TELESCOP']=params['telescope']
            hdu.header['FILENAME']=fitspath.name
            hdu.header['HISTORY']=f'{tiffile}'
            hdu.header['HISTORY']=f'scoobi'

def thumbgen(fitsfile, out_folder=None, thumb_folder=thumbnails, force=False):
    """
    Creates thumbnails for the fits file provided in the fits_folder path.
    depends on build_foldername 
    else if thumb_folder is specified with out_folder jpgfile is created accordingly.
    ex. out_folder,thumb_foler='/data/solardata/','Thumbnails/'

    """
    if fitsfile:        
        jpgfile_name=f"{Path(fitsfile).stem}.jpg"
        if out_folder is None:out_folder=Path(fitsfile).parent
        jpgfile=f'{out_folder}/{thumb_folder}/{jpgfile_name}'
        if not Path(jpgfile).exists(): Path(jpgfile).parent.mkdir(parents=True,exist_ok=True)
        if not Path(jpgfile).exists() or force: subprocess.run(["convert", fitsfile,"-linear-stretch","1x1","-resize", "56%", jpgfile ])
        return jpgfile

def thumbgen_bulk(fits_folder, thumb_folder=thumbnails, thumb_root=thumbnails_root, force=False, **kwargs):
    """
    calls thumb_gen for converting fits to jpg and then copies to a directory specified by :thumb_root:
    """
    
    allfile=search_file(f"{fits_folder}/",'.fits', recursive=True)
    for fitsfile in allfile:
        jpgfile=thumbgen(fitsfile,thumb_folder=thumb_folder,force=force)

        outfolder_root=build_foldername(Path(fitsfile).name,createdir=False, destfolder='/')
        jpgfile_root=f"{thumb_root}/{Path(outfolder_root).parent}/{Path(jpgfile).name}"
        
        if not Path(jpgfile_root).exists() or force: 
            Path(jpgfile_root).parent.mkdir(parents=True,exist_ok=True)
            shutil.copy(str(jpgfile), str(jpgfile_root))

            
def fits2fits_bulk(fitsfolderlist, **kwargs):
    """
    takes input as list of string fits folder paths and executes fits_to_fits() using for loop to each folder path.
    see *fits_to_fits()* for the list of parameters.
    If fits to fits conversion fails, the file is saved in "corrupt/" (can be changed from call) folder at the tiff source folder.
    """
    params={'failed_folder':'corrupt/'}
    params.update(kwargs)
        
    if isinstance(fitsfolderlist,list):
        for fitsfolder in fitsfolderlist:
            listfits=search_file(fitsfolder,'.fits', recursive=True)
            for f in listfits:
                if not 'corrupt' in f:
                    params['failed_folder']=f"{Path(f).parent}/corrupt/"
                    try:
                        fits_to_fits(f, **kwargs)
                    except Exception as e:
                        try:
                            # if Path(f).stat().st_size <= desiredsize:
                            print(e)
                            Path(params['failed_folder']).mkdir(parents=True,exist_ok=True)
                            shutil.copy(str(f), str(f"{params['failed_folder']}/{Path(f).name}"))
                        except:
                            with open('failed.scoobi','+a') as sf:
                                sf.write(f"{f}\n")
    else:
        raise Exception(f'Is "{fitsfolderlist}" a list?')           

def tif2fits_bulk(tiffolderlist, **kwargs):
    """
    takes input as list of string tiff folder paths and executes tif to fits using for loop to each folder path.
    see *tif_to_fits()* for the list of parameters.
    If tif to fits conversion fails, the file is saved in "corrupt/" (can be changed from call) folder at the tiff source folder.
    """
    params={'failed_folder':'corrupt/'}
    params.update(kwargs)
        
    if isinstance(tiffolderlist,list):
        for tifffolder in tiffolderlist:
            listtiff=search_file(tifffolder,'.tif', recursive=True)
            for f in listtiff:
                if not 'corrupt' in f:
                    params['failed_folder']=f"{Path(f).parent}/corrupt/"
                    try:
                        tif_to_fits(f, **kwargs)
                    except:
                        try:
                            # if Path(f).stat().st_size <= desiredsize:
                            Path(params['failed_folder']).mkdir(parents=True,exist_ok=True)
                            shutil.copy(str(f), str(f"{params['failed_folder']}/{Path(f).name}"))
                        except:
                            with open('failed.scoobi','+a') as sf:
                                sf.write(f"{f}\n")
    else:
        raise Exception(f'Is "{tiffolderlist}" a list?')
        
#log generation
def compare_datetime(**kwargs):
    params={
        'fits_folder':'/data/solar_data/processed/', 
        'csv_file':'/data/solar_data/processed/scooby_logs/compare_datetime.csv',
        'tiff_folder':'/data/solar_data/raw/SOLAR_DATA_2012/', 
        'month':['Jan','Feb','Mar', 'Apr','May', 'June','July','Aug','Sept','Oct','Nov','Dec'], 
        'days':range(1,32)
    }
    params.update(kwargs)
    
    sfolder=None
    month_dict=['Jan','Feb','Mar', 'Apr','May', 'June','July','Aug','Sept','Oct','Nov','Dec']
    allmonthfolders=glob(f"{params['fits_folder']}*")

    for m in params['month']:
        for mm in allmonthfolders: 
                if m in mm:
                    sfolder=f'{mm}'
        
        for day in params['days']:
            
            d_str=str(day).zfill(2)
        
            list_fitsfile=search_file(f'{sfolder}/{d_str}/', '.fits', True)
            tlist_fitsfile=len(list_fitsfile)
            
            csv_data=[]    
            for fitsfile in list_fitsfile:
                
                hdul=fits.open(fitsfile)
                try:
                    d=hdul[0].header['DATE-OBS']
                    fl=hdul[0].header['FILENAME']
                except KeyError:
                    d=hdul[1].header['DATE-OBS']
                    fl=hdul[1].header['FILENAME']
                
                
                # build name for dated folder in raw_data
                dm=d.split('-')    
                datedfolder=str(int(dm[2][:2]))+str(month_dict[int(dm[1])-1])
                list_tiffile=search_file(f'{params["tiff_folder"]}{m}/{datedfolder}',fl, recursive=True,dontknowfoldername=True)
                ltf_datetime_found,ltf_found='NA',f'missing /{datedfolder.lower()}' # init with NA if missing

                if not len(list_tiffile):
                    list_tiffile=search_file(f'{params["tiff_folder"]}{m.upper()}/{datedfolder}',fl, recursive=True,dontknowfoldername=True)
                if not len(list_tiffile):
                    list_tiffile=search_file(f'{params["tiff_folder"]}{m.upper()}/{datedfolder.lower()}',fl, recursive=True,dontknowfoldername=True)
                
                
                for ltf in list_tiffile:
                    if f'/{datedfolder.lower()}' in ltf.lower():
                        # try:
                        #     ltf_datetime=str(Time.strptime(str(read_tif(ltf)['Image DateTime'].values),'%m/%d/%Y %I:%M:%S.%f %p',scale='utc', format='fits'))
                        # except:
                        try:
                            ltf_datetime=str(read_tif(ltf)['Image DateTime'].values)
                        except:
                            ltf_datetime='NA'
                        ltf_datetime_found,ltf_found=ltf_datetime,ltf
                csv_data.append([Path(fitsfile).name,d,fl,ltf_datetime_found,ltf_found])
                
            tcsv_data=len(csv_data)
            if tcsv_data:
                with open(params['csv_file'], 'a') as cfile:
                    cw=csv.writer(cfile)
                    cw.writerows(csv_data)
                    # print(f"ww={tcsv_data}/tt={tlist_fitsfile}")

def compare_datetime_nofolder(**kwargs):
    params={
        'fits_folder':'/data/solar_data/processed/', 
        'csv_file':'/data/solar_data/processed/scooby_logs/compare_datetime.csv',
        'tiff_folder':'/data/solar_data/raw/SOLAR_DATA_2012/',  
        
    }
    params.update(kwargs)
    
    list_fitsfile=search_file(f'{params["fits_folder"]}/', '.fits', True)
    tlist_fitsfile=len(list_fitsfile)
    
    csv_data=[]    
    for fitsfile in list_fitsfile:
        
        hdul=fits.open(fitsfile)
        try:
            d=hdul[0].header['DATE-OBS']
            fl=hdul[0].header['FILENAME']
        except KeyError:
            d=hdul[1].header['DATE-OBS']
            fl=hdul[1].header['FILENAME']
        
        
        # build name for dated folder in raw_data
        dm=d.split('-')    
        
        list_tiffile=search_file(f'{params["tiff_folder"]}',fl, recursive=True,dontknowfoldername=True)
        ltf_datetime_found,ltf_found='NA',f'missing /{fl}' # init with NA if missing

        if not len(list_tiffile):
            list_tiffile=search_file(f'{params["tiff_folder"]}',fl, recursive=True,dontknowfoldername=False)
        
        for ltf in list_tiffile:
            try:
                ltf_datetime=str(read_tif(ltf)['Image DateTime'].values)
            except:
                ltf_datetime='NA'
            ltf_datetime_found,ltf_found=ltf_datetime,ltf
        csv_data.append([Path(fitsfile).name,d,fl,ltf_datetime_found,ltf_found])
        
    tcsv_data=len(csv_data)
    if tcsv_data:
        with open(params['csv_file'], 'a') as cfile:
            cw=csv.writer(cfile)
            cw.writerows(csv_data)



import argparse
from scoobi import compare_datetime, read_tif, Time, tif2fits_bulk, tif_to_fits, compare_datetime_nofolder, thumbgen_bulk,fits2fits_bulk,fits_to_fits
from collections import defaultdict
from pathlib import Path
from scoobi.config import rootfolder,processedfolder,rawfolder,config

def read_configfile(filepath):
    with open(filepath,'r') as f:
        params = defaultdict(list)
        pr=f.read().splitlines()
        for i in range(len(pr)):
            if '#' in pr[i]:
                continue
            elif '=' in pr[i]:
                k,v=pr[i].split('=')
                if 'month' in k:
                    params[k]=v.split(',')
                elif 'days' in k:
                    dd=v.split(',')
                    params[k]=range(int(dd[0]),int(dd[1])) 
                else:
                    try:
                        v=int(v)
                    except:
                        v=str(v)
                        if 'True' in v: v=v.lower()=='true'
                        elif 'False' in v:v=v.lower()=='true'
                    
                    params[k]=v
    return params

def default_params():
    params= {'fits_folder':processedfolder,
        'csv_file':f'{rootfolder}log.csv',
        'tiff_folder':rawfolder,
        'raw_folder':rawfolder, 
        'rootfolder':rootfolder,
        'month':'Jan,Feb,Mar,Apr,May,June,July,Aug,Sept,Oct,Nov,Dec', 
        'days':'1,32'}
    params.update(config)
    return params

def create_config(params, out='config.py'):
    # if Path(out).exists():
    
    with open(out, 'w') as o:
        for k,v in params.items():
            if isinstance(v,list) : 
                o.write(f'{k}={",".join(v)}\n')
            elif isinstance(v, range):
                o.write(f'{k}={v.start},{v.stop}\n')
            else:
                o.write(f'{k}={v}\n')
    return f'configfile:{out}'
    

parser = argparse.ArgumentParser('scoobi',description="""
Solar Conventionality-based Organizing Observation data ( SCOOBI )""", formatter_class=argparse.RawDescriptionHelpFormatter)

# pa = parser.add_subparsers(help='compare datetimes') # subparser is used like $pip "install"

parser.add_argument('-f','--fits_folder', help='source') #positionals
parser.add_argument('-t','--tiff_folder', help='RAW TIFF folder path')
parser.add_argument('-r','--raw_folder', help='RAW FITS folder path')
parser.add_argument('-o','--output_file', help='complete path for output csv or log file.')
parser.add_argument('-c','--config_file', help='complete path for config file with inputs', default='.config')
# parser.add_argument('-hdr','--header', help='Input header to inject to the FITS file.')

headers=parser.add_argument_group('header', """Parameters for injecting and printing headers.
[[Work in Progress]]""")
headers.add_argument('-hdr','--header', type=str, help="Comma separated input headers to inject to the raw file.", required=False)
headers.add_argument('-hf', '--header-file', type=str, help="Input header from path to inject to the raw file.", required=False)
headers.add_argument('-hfo', '--header-file-output', type=str, help="Header output with the correct padding.", required=False)

parser.add_argument('-mm', '--month', type=str, help="""comma separated month names""")
parser.add_argument('-dd', '--days', type=str, default='1,32', help="""comma separated month names for range(a,b) 
Ex --days='1,32'""")

parser.add_argument('--no-datedfolder', action='store_true', dest='no_datedfolder',help='if not needed to create dated folder') 
parser.add_argument('-rt','--read-time', action='store_true', dest='rt',help='reads time from a tiff file') 
parser.add_argument('-ct','--compare-time', action='store_true', dest='ct',help='compares time from a fits file to a tiff file') 
parser.add_argument('-rc','--read-config',action='store_true',dest='rc', help=" read config, bool")
parser.add_argument('-cc','--create-config',action='store_true',dest='cc', help=" create config, if called with rc would modify from and to the CONFIG_FILE path")
parser.add_argument('-do','--do-conversion',action='store_true',dest='do', help=" create fits from tiff/fits raw file and take care of the folder structure. Requires tiff_folder or raw_folder path")
parser.add_argument('-th','--thumbnail', action='store_true', dest='th',help='True/False for creating thumbnails; The fits folder path is required but should be more specific e.g atleaset including the /processed; should not be used with -do')

args=parser.parse_args()
def cli():
    params=default_params()
    header = defaultdict(list)
    if args.fits_folder: params['fits_folder']=args.fits_folder
    if args.output_file: params['csv_file']=args.output_file
    if args.tiff_folder: params['tiff_folder']=args.tiff_folder
    if args.raw_folder: params['raw_folder']=args.raw_folder
    if args.header_file_output: print(args.header_file_output)
    if args.header: 
        hdrin=str(args.header).replace(' ','')
        hdrdict=hdrin.split(',')
        for i in range(len(hdrdict)):
            if '=' in hdrdict[i]:
                hdrk,hdrv=hdrdict[i].split('=')
                try:
                    hdrv=int(hdrv)
                except:
                    try:
                        hdrv=float(hdrv)
                    except:
                        hdrv=str(hdrv).strip()
                header[hdrk]=hdrv  
        print(header)

    mm=args.month or params['month']
    dd=args.days #already has default values
    configfile=args.config_file or '.config'

    if dd: dd=str(dd).split(',')
    if mm: params['month']=mm.split(',')
    params['days']=range(int(dd[0]),int(dd[1]))    
    params={k: v for k, v in params.items() if v is not None}
    if Path(configfile).exists() and args.rc: params.update(read_configfile(configfile))

    if args.ct and not args.no_datedfolder: compare_datetime(**params)
    if args.no_datedfolder and args.ct: compare_datetime_nofolder(**params)
    if args.rc: 
        params.update(read_configfile(configfile))
        print(params)
    if args.cc: print(create_config(params,configfile))
    if args.rt: print(str(read_tif(params['tiff_folder'])['Image DateTime'].values))
    if args.do: 
        if args.tiff_folder:
            if '.tif' not in params['tiff_folder']: 
                print(tif2fits_bulk(params['tiff_folder'].split(','), destfolder=params['fits_folder']))
            else:
                print(tif_to_fits(params['tiff_folder'], destfolder=params['fits_folder']))
        if args.raw_folder:
            if not any(fts in params['raw_folder'] for fts in ['.fits', '.fts']): 
                print(fits2fits_bulk(params['raw_folder'].split(','), destfolder=params['fits_folder']))
            else:
                print(fits_to_fits(params['raw_folder'], destfolder=params['fits_folder']))
    if args.th: print(thumbgen_bulk(params['fits_folder'],rootfolder=params['rootfolder']))
        
if __name__=='__main__':
    cli()
